@@ -399,7 +399,10 @@ class AsyncServer(server.Server):
             s.connected = True
             headers = None
             if self.cookie:
-                headers = [('Set-Cookie', self.cookie + '=' + sid)]
+                headers = [(
+                    'Set-Cookie',
+                    self.cookie + '=' + sid + '; path=/; SameSite=Lax'
+                )]
             try:
                 return self._ok(await s.poll(), headers=headers, b64=b64,
                                 jsonp_index=jsonp_index)
@@ -460,7 +463,12 @@ class AsyncServer(server.Server):
                     if not socket.closing and not socket.closed:
                         await socket.check_ping_timeout()
                     await self.sleep(sleep_interval)
-            except (SystemExit, KeyboardInterrupt, asyncio.CancelledError):
+            except (
+                SystemExit,
+                KeyboardInterrupt,
+                asyncio.CancelledError,
+                GeneratorExit,
+            ):
                 self.logger.info('service task canceled')
                 break
             except:
